// File: Crypto.java
// Base class for encryption interceptors

package Encryption;

import com.visigenic.vbroker.interceptor.*;
import com.visigenic.vbroker.IOP.*;
import com.visigenic.vbroker.GIOP.*;
import com.visigenic.vbroker.orb.*;
import org.omg.CORBA.*;
import org.omg.CORBA.portable.*;

public class Crypto
{
  protected static int ENCRYPTION_KEY_REQUEST = 4;
  protected static int ENCRYPTION_KEY_REPLY = 5;

  protected String _name;
  protected com.visigenic.vbroker.orb.ORB _orb;


  // Constructor: we need a reference to the ORB in order to create new
  // Giop streams
  public Crypto(String name, com.visigenic.vbroker.orb.ORB orb)
  {
    _orb = orb;
    _name = name;
  }


  // interceptPrint: print out trace information
  protected void interceptPrint(String value)
  {
    System.out.println(_name + ": " + value);
  }

  // ===========================================================================
  // adjustBuffer: helper function which, given buffers, starting and ending
  // points and an adjustment value, adds the adjustment to each byte in the 
  // buffer between startOffset and endOffset.  The offset() of src and dst must
  // already point to the place to start reading and writing, respectively.
  // ===========================================================================
  private GiopOutputStream adjustBuffer(GiopInputStream src, GiopOutputStream dst,
					int adjustment, int startOffset, 
					int endOffset)
  {
    for (int i = startOffset; i < endOffset; i++) {
      // read_char() actually reads one byte (not two)
      char charMod = src.read_char();
      char tmp = (char)(charMod + (byte)adjustment);
      dst.write_char(tmp);
    }

    return dst;
  }


  // ===========================================================================
  // encryptBuffer: given an adjustment, a buffer, startOffset, and length,
  // encrypts the buffer and hands it back.  It is important to note that this
  // particular algorithm inserts the key as the first element in the new
  // data body, and then the encrypted data.  The receiver of this buffer will
  // then be responsible for extracting the adjustment key to decrypt the rest
  // of the buffer.
  // ===========================================================================
  private GiopOutputStream encryptBuffer(int adjustment, 
					 GiopInputStream buffer,
					 int startOffset, int length)
  {
    GiopOutputStream ostrm = 
      (GiopOutputStream)_orb.newGiopOutputStream();

    ostrm.write_octet_array(buffer.bytes(), 0, buffer.bytes().length);
    ostrm.offset(startOffset);
    buffer.offset(startOffset);

    ostrm.write_long(adjustment);

    ostrm = adjustBuffer(buffer, ostrm, adjustment, 
			 ostrm.offset(), ostrm.offset() + length);

    ostrm.dataOffset(startOffset);

    return ostrm;
  }


  // ===========================================================================
  // decryptBuffer: given a stream, beginning and ending point, extracts the 
  // adjustment key from the first part of the stream and then uses it to decrypt
  // the rest.  Performs the inverse of encryptBuffer().
  // ===========================================================================
  private GiopOutputStream decryptBuffer(GiopInputStream buffer, int startOffset,
					 int endOffset)
  {
    GiopOutputStream ostrm = 
      (GiopOutputStream)_orb.newGiopOutputStream();
    ostrm.write_octet_array(buffer.bytes(), 0, buffer.bytes().length);
    ostrm.offset(startOffset);
    buffer.offset(startOffset);

    int adjustment = -(buffer.read_long());

    ostrm = adjustBuffer(buffer, ostrm, adjustment, buffer.offset(), 
			 endOffset);

    ostrm.dataOffset(startOffset);

    return ostrm;
  }
					 

  // ===========================================================================
  // Encryption for requests and replies: accessed from send and 
  // receive calls.
  // 
  // IMPORTANT: the stream returned from the interceptor in a send 
  // or receive call must have the dataOffset field set to the beginning
  // of the data body (following all headers) and the offset field set 
  // to the end of the message.  This is absolutely necessary so that 
  // the ORB can tell if you have increased the length of your message.
  // Set these using offset() and dataOffset() settors.
  // ===========================================================================
  public OutputStream encrypt (OutputStream buf,
			       int adjustment) 
  { 
    GiopOutputStream ostr = (GiopOutputStream)buf;
    int dataStart = ostr.dataOffset();
    int dataEnd = ostr.offset();

    GiopInputStream istr = (GiopInputStream)_orb.newGiopInputStream(ostr.bytes());

    GiopOutputStream ret = encryptBuffer(adjustment, istr, dataStart,
					 dataEnd - dataStart);

    // encryptBuffer was responsible for setting the offsets correctly
    return ret;    
  }


  public InputStream decrypt (InputStream buf) 
  { 
    GiopInputStream istr = (GiopInputStream)buf;
    int dataEnd = istr.offset();
    int dataStart = istr.dataOffset();

    GiopOutputStream ostr = decryptBuffer(istr, dataStart, dataEnd);

    GiopInputStream ret_istr = (GiopInputStream)ostr.create_input_stream();

    // We've created a new stream, so we need to set the offsets.  Decrypt
    // buffer set the ostr offsets correctly, so we need to just copy those
    // over to the new stream
    ret_istr.offset(ostr.offset());
    ret_istr.dataOffset(ostr.dataOffset());

    return ret_istr;
  }
}




